@@ -50,7 +50,7 @@ constants <- c("TRUE", "FALSE", "NULL",
 #' Complete language constants
 #' @keywords internal
 constant_completion <- function(token) {
-    consts <- constants[startsWith(constants, token)]
+    consts <- constants[match_with(constants, token)]
     completions <- lapply(consts, function(const) {
         list(label = const,
             kind = CompletionItemKind$Constant,
@@ -64,7 +64,7 @@ constant_completion <- function(token) {
 #' @keywords internal
 package_completion <- function(token) {
     installed_packages <- .packages(all.available = TRUE)
-    token_packages <- installed_packages[startsWith(installed_packages, token)]
+    token_packages <- installed_packages[match_with(installed_packages, token)]
     completions <- lapply(token_packages, function(package) {
         list(label = package,
             kind = CompletionItemKind$Module,
@@ -95,7 +95,7 @@ arg_completion <- function(uri, workspace, point, token, funct, package = NULL,
                 last_def <- all_defs[[length(all_defs)]]
                 func_line1 <- as.integer(xml_attr(last_def, "line1"))
                 args <- xml_text(xml_find_all(last_def, "SYMBOL_FORMALS"))
-                token_args <- args[startsWith(args, token)]
+                token_args <- args[match_with(args, token)]
                 token_data <- list(
                     type = "parameter",
                     funct = funct,
@@ -112,8 +112,13 @@ arg_completion <- function(uri, workspace, point, token, funct, package = NULL,
 
     if (!is.null(package)) {
         args <- names(workspace$get_formals(funct, package, exported_only = exported_only))
+
+        if (package == "base" && funct == "options") {
+            args <- c(args, names(.Options))
+        }
+
         if (is.character(args)) {
-            token_args <- args[startsWith(args, token)]
+            token_args <- args[match_with(args, token)]
             token_data <- list(
                 type = "parameter",
                 funct = funct,
@@ -122,16 +127,16 @@ arg_completion <- function(uri, workspace, point, token, funct, package = NULL,
         }
     }
 
-    completions <- lapply(token_args, function(arg) {
+    completions <- .mapply(function(arg, sort_text) {
         list(label = arg,
             kind = CompletionItemKind$Variable,
             detail = "parameter",
-            sortText = paste0(sort_prefixes$arg, arg),
+            sortText = sort_text,
             insertText = paste0(arg, " = "),
             insertTextFormat = InsertTextFormat$PlainText,
             data = token_data
         )
-    })
+    }, list(token_args, sprintf("%s%03d", sort_prefixes$arg, seq_along(token_args))), NULL)
 
     completions
 }
@@ -140,7 +145,7 @@ arg_completion <- function(uri, workspace, point, token, funct, package = NULL,
 ns_function_completion <- function(ns, token, exported_only, snippet_support) {
     nsname <- ns$package_name
     functs <- ns$get_symbols(want_functs = TRUE, exported_only = exported_only)
-    functs <- functs[startsWith(functs, token)]
+    functs <- functs[match_with(functs, token)]
     if (nsname == WORKSPACE) {
         tag <- "[workspace]"
         sort_prefix <- sort_prefixes$workspace
@@ -179,7 +184,7 @@ ns_function_completion <- function(ns, token, exported_only, snippet_support) {
 imported_object_completion <- function(workspace, token, snippet_support) {
     completions <- NULL
     for (object in workspace$imported_objects$keys()) {
-        if (!startsWith(object, token)) {
+        if (!match_with(object, token)) {
             next
         }
         nsname <- workspace$imported_objects$get(object)
@@ -246,7 +251,7 @@ workspace_completion <- function(workspace, token,
                 exported_only = TRUE, snippet_support = snippet_support)
 
             nonfuncts <- ns$get_symbols(want_functs = FALSE, exported_only = TRUE)
-            nonfuncts <- nonfuncts[startsWith(nonfuncts, token)]
+            nonfuncts <- nonfuncts[match_with(nonfuncts, token)]
             nonfuncts_completions <- lapply(nonfuncts, function(object) {
                 list(label = object,
                      kind = CompletionItemKind$Field,
@@ -258,7 +263,7 @@ workspace_completion <- function(workspace, token,
                      ))
             })
             lazydata <- ns$get_lazydata()
-            lazydata <- lazydata[startsWith(lazydata, token)]
+            lazydata <- lazydata[match_with(lazydata, token)]
             lazydata_completions <- lapply(lazydata, function(object) {
                 list(label = object,
                      kind = CompletionItemKind$Field,
@@ -282,7 +287,7 @@ workspace_completion <- function(workspace, token,
                 exported_only = FALSE, snippet_support = snippet_support)
 
             nonfuncts <- ns$get_symbols(want_functs = FALSE, exported_only = FALSE)
-            nonfuncts <- nonfuncts[startsWith(nonfuncts, token)]
+            nonfuncts <- nonfuncts[match_with(nonfuncts, token)]
             nonfuncts_completions <- lapply(nonfuncts, function(object) {
                 list(label = object,
                      kind = CompletionItemKind$Field,
@@ -333,7 +338,7 @@ scope_completion <- function(uri, workspace, token, point, snippet_support = NUL
     scope_symbol_nodes <- xml_find_all(enclosing_scopes, scope_completion_symbols_xpath)
     scope_symbol_names <- xml_text(scope_symbol_nodes)
     scope_symbol_lines <- as.integer(xml_attr(scope_symbol_nodes, "line1"))
-    scope_symbol_selector <- startsWith(scope_symbol_names, token)
+    scope_symbol_selector <- match_with(scope_symbol_names, token)
 
     scope_symbol_names <- rev(scope_symbol_names[scope_symbol_selector])
     scope_symbol_lines <- rev(scope_symbol_lines[scope_symbol_selector])
@@ -359,7 +364,7 @@ scope_completion <- function(uri, workspace, token, point, snippet_support = NUL
     scope_funct_nodes <- xml_find_all(enclosing_scopes, scope_completion_functs_xpath)
     scope_funct_names <- xml_text(scope_funct_nodes)
     scope_funct_lines <- as.integer(xml_attr(scope_funct_nodes, "line1"))
-    scope_funct_selector <- startsWith(scope_funct_names, token)
+    scope_funct_selector <- match_with(scope_funct_names, token)
 
     scope_funct_names <- rev(scope_funct_names[scope_funct_selector])
     scope_funct_lines <- rev(scope_funct_lines[scope_funct_selector])
@@ -410,9 +415,25 @@ token_completion <- function(uri, workspace, token, exclude = NULL) {
         return(list())
     }
 
-    symbols <- unique(xml_text(xml_find_all(xdoc,
-        "//SYMBOL | //SYMBOL_SUB | //SYMBOL_FORMALS | //SYMBOL_FUNCTION_CALL")))
-    symbols <- symbols[startsWith(symbols, token)]
+    token_quote <- xml_single_quote(token)
+
+    symbols <- xml_text(xml_find_all(xdoc,
+        glue("//*[
+            (self::SYMBOL[preceding-sibling::OP-DOLLAR] or self::SYMBOL_SUB) and
+            starts-with(text(),'{token_quote}')]",
+            token_quote = token_quote
+        )
+    ))
+
+    if (nzchar(token)) {
+        symbols <- c(symbols, xml_text(xml_find_all(xdoc,
+            glue("//*[(self::SYMBOL or self::SYMBOL_SUB or self::SYMBOL_FORMALS or self::SYMBOL_FUNCTION_CALL) and
+                starts-with(text(),'{token_quote}')]",
+                token_quote = token_quote
+            )
+        )))
+    }
+
     symbols <- setdiff(symbols, exclude)
     token_completions <- lapply(symbols, function(symbol) {
         list(
@@ -434,6 +455,8 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
                 items = list()
             )))
     }
+
+    t0 <- Sys.time()
     snippet_support <- isTRUE(capabilities$completionItem$snippetSupport) &&
         getOption("languageserver.snippet_support", TRUE)
 
@@ -478,10 +501,32 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
         )
     }
 
+    init_count <- length(completions)
+    nmax <- getOption("languageserver.max_completions", 200)
+
+    if (init_count > nmax) {
+        isIncomplete <- TRUE
+        label_text <- vapply(completions, "[[", character(1), "label")
+        sort_text <- vapply(completions, "[[", character(1), "sortText")
+        order <- order(!startsWith(label_text, token), sort_text)
+        completions <- completions[order][seq_len(nmax)]
+    } else {
+        isIncomplete <- FALSE
+    }
+
+    t1 <- Sys.time()
+
+    logger$info("completions: ", list(
+        init_count = init_count,
+        final_count = length(completions),
+        time = as.numeric(t1 - t0),
+        isIncomplete = isIncomplete
+    ))
+
     Response$new(
         id,
         result = list(
-            isIncomplete = FALSE,
+            isIncomplete = isIncomplete,
             items = completions
         )
     )
