@@ -21,6 +21,7 @@ Workspace <- R6::R6Class("Workspace",
 
         startup_packages = NULL,
         loaded_packages = NULL,
+        help_cache = NULL,
 
         initialize = function(root) {
             self$root <- root
@@ -41,6 +42,7 @@ Workspace <- R6::R6Class("Workspace",
             for (pkgname in self$loaded_packages) {
                 self$namespaces$set(pkgname, PackageNamespace$new(pkgname))
             }
+            self$help_cache <- collections::dict()
         },
 
         load_package = function(pkgname) {
@@ -146,9 +148,31 @@ Workspace <- R6::R6Class("Workspace",
             )
 
             if (length(hfile) > 0) {
-                enc2utf8(repr::repr_text(hfile))
-            } else {
-                NULL
+                key <- as.character(hfile)
+                if (self$help_cache$has(key)) {
+                    return(self$help_cache$get(key))
+                } else {
+                    result <- NULL
+
+                    if (requireNamespace("rmarkdown", quietly = TRUE) &&
+                        rmarkdown::pandoc_available()) {
+                        html <- enc2utf8(repr::repr_html(hfile))
+                        # Make header look prettier:
+                        pattern <- "<table.*?<td>(.*?)\\s*{(.*?)}<\\/td>.*?<\\/table>\\n*<h2>\\s*(.*?)\\s*<\\/h2>"
+                        replacement <- "<b>\\1</b> <i>{\\2}</i><p>\\3</p><hr/>"
+                        html <- gsub(pattern, replacement, html, perl = TRUE)
+                        result <- html_to_markdown(html)
+                    }
+
+                    if (is.null(result)) {
+                        result <- enc2utf8(repr::repr_text(hfile))
+                    }
+
+                    if (!is.null(result)) {
+                        self$help_cache$set(key, result)
+                    }
+                    return(result)
+                }
             }
         },
 
