@@ -1,5 +1,3 @@
-context("Test Completion")
-
 test_that("Simple completion works", {
     skip_on_cran()
     client <- language_client()
@@ -60,6 +58,66 @@ test_that("Simple completion works", {
     expect_length(result$items %>% keep(~ .$label == "mtcars"), 1)
 })
 
+test_that("Simple completion is case insensitive", {
+    skip_on_cran()
+    client <- language_client()
+
+    temp_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(
+        c(
+            "STR",
+            "File.c",
+            "fs::PATH",
+            "foo$sol",
+            ".mac",
+            "grdev",
+            "tru",
+            "utils:::.gethelp",
+            "utils::.gethelp",
+            "utils::osver",
+            "datasets::MTCAR"
+        ),
+        temp_file)
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(temp_file, c(0, 3))
+
+    expect_length(result$items %>% keep(~ .$label == "strsplit"), 1)
+    expect_length(result$items %>% keep(~ .$label == "strrep"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(1, 6))
+    expect_length(result$items %>% keep(~ .$label == "file.choose"), 1)
+    expect_length(result$items %>% keep(~ .$label == "file.create"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(2, 8))
+    expect_true("path_real" %in% (result$items %>% map_chr(~ .$label)))
+
+    result <- client %>% respond_completion(temp_file, c(3, 7))
+    expect_length(result$items %>% discard(~ .$kind == CompletionItemKind$Text), 0)
+
+    result <- client %>% respond_completion(temp_file, c(4, 4))
+    expect_length(result$items %>% keep(~ .$label == ".Machine"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(5, 5))
+    expect_length(result$items %>% keep(~ .$label == "grDevices"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(6, 3))
+    expect_length(result$items %>% keep(~ .$label == "TRUE"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(7, 16))
+    expect_length(result$items %>% keep(~ .$label == ".getHelpFile"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(8, 15))
+    expect_length(result$items, 0)
+
+    result <- client %>% respond_completion(temp_file, c(9, 12))
+    expect_length(result$items %>% keep(~ .$label == "osVersion"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(10, 15))
+    expect_length(result$items %>% keep(~ .$label == "mtcars"), 1)
+})
+
 test_that("Completion of function arguments works", {
     skip_on_cran()
     client <- language_client()
@@ -88,6 +146,95 @@ test_that("Completion of function arguments works", {
     expect_length(arg_items, 0)
 })
 
+test_that("Completion of function arguments is case insensitive", {
+    skip_on_cran()
+    client <- language_client()
+
+    temp_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(
+        c(
+            "str(OBJ",
+            "utils::str(OBJ",
+            "str(stats::O"
+        ),
+        temp_file)
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(temp_file, c(0, 6))
+    arg_items <- result$items %>% keep(~ .$label == "object")
+    expect_length(arg_items, 1)
+
+    result <- client %>% respond_completion(temp_file, c(1, 14))
+    arg_items <- result$items %>% keep(~ .$label == "object")
+    expect_length(arg_items, 1)
+
+    result <- client %>% respond_completion(temp_file, c(2, 12))
+    arg_items <- result$items %>% keep(~ .$label == "object")
+    expect_length(arg_items, 0)
+})
+
+test_that("Completion of options works", {
+    skip_on_cran()
+    client <- language_client()
+
+    temp_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(
+        c(
+            "options(sci",
+            "options(scipen = 999, useFancy"
+        ),
+        temp_file)
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(temp_file, c(0, 11))
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, "scipen")
+
+    result <- client %>% respond_completion(temp_file, c(1, 30))
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, "useFancyQuotes")
+})
+
+test_that("Completion of function arguments preserves the order of arguments", {
+    skip_on_cran()
+    client <- language_client()
+
+    temp_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(
+        c(
+            "eval(",
+            "formatC(",
+            "print.default("
+        ),
+        temp_file)
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(temp_file, c(0, 5))
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, names(formals(eval)))
+
+    result <- client %>% respond_completion(temp_file, c(1, 8))
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, names(formals(formatC)))
+
+    result <- client %>% respond_completion(temp_file, c(2, 14))
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, names(formals(print.default)))
+})
+
 test_that("Completion of local function arguments works", {
     skip_on_cran()
     client <- language_client()
@@ -118,6 +265,34 @@ test_that("Completion of local function arguments works", {
     expect_length(arg_items, 1)
 })
 
+test_that("Completion of user function arguments preserves the order of arguments", {
+    skip_on_cran()
+    client <- language_client()
+
+    temp_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(
+        c(
+            "test <- function(var3, var2, var1) {",
+            "  var1 + var2 + var3",
+            "}",
+            "test(",
+            ")"
+        ),
+        temp_file)
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(
+        temp_file, c(3, 5),
+        retry_when = function(result) length(result) == 0 || length(result$items) == 0
+    )
+    arg_items <- result$items %>%
+        keep(~ identical(.$data$type, "parameter")) %>%
+        map_chr(~ .$label)
+    expect_identical(arg_items, c("var3", "var2", "var1"))
+})
+
+
 test_that("Completion of user function works", {
     skip_on_cran()
     client <- language_client()
