@@ -1,5 +1,3 @@
-context("Test Symbol")
-
 test_that("Document Symbol works", {
     skip_on_cran()
     client <- language_client()
@@ -19,7 +17,7 @@ test_that("Document Symbol works", {
     client %>% did_save(defn_file)
     result <- client %>% respond_document_symbol(defn_file)
 
-    expect_equal(result %>% map_chr(~ .$name) %>% sort(), c("f", "g", "p", "m") %>% sort())
+    expect_setequal(result %>% map_chr(~ .$name), c("f", "g", "p", "m"))
     expect_equivalent(
         result %>% detect(~ .$name == "f") %>% pluck("location", "range"),
         range(position(0, 0), position(2, 1))
@@ -38,6 +36,43 @@ test_that("Document Symbol works", {
     )
 })
 
+test_that("Recognize symbols created by delayedAssign()/assign()/makeActiveBinding()", {
+    skip_on_cran()
+    client <- language_client()
+
+    defn_file <- withr::local_tempfile(fileext = ".R")
+    writeLines(c(
+        "delayedAssign('d1', 1)",
+        "delayedAssign(value = function() 2, x = 'd2')",
+        "base::delayedAssign(value = '3', 'd3')",
+        "delayedAssign(('d4'), 4)",
+        "delayedAssign('d5', 5, assign.env = globalenv())",
+        "delayedAssign('d6', 6, assign.env = emptyenv())",
+        "delayedAssign('d7', 7, assign.env = parent.frame(1))",
+        "makeActiveBinding('a1', function() 1, environment())",
+        "makeActiveBinding(function() '2', sym = 'a2')",
+        "base::makeActiveBinding(",
+        "   fun = function() stop('3'),",
+        "   sym = 'a3'",
+        ")",
+        "makeActiveBinding(('a4'), function() 4, environment())",
+        "makeActiveBinding('a5', function() 5, .GlobalEnv)",
+        "makeActiveBinding('a6', function() 6, new.env())",
+        "assign(value = '1', x = 'assign1')",
+        "assign('assign2', 2, pos = -1L)",
+        "assign('assign3', 3, pos = environment())",
+        "assign('assign4', 4, pos = new.env())"
+    ), defn_file)
+
+    client %>% did_save(defn_file)
+    result <- client %>% respond_document_symbol(defn_file)
+
+    expect_setequal(
+        result %>% map_chr(~ .$name),
+        c("d1", "d2", "d3", "d5", "d7", "a1", "a2", "a3", "a5", "assign1", "assign2", "assign3")
+    )
+})
+
 test_that("Document section symbol works", {
     skip_on_cran()
     client <- language_client(capabilities = list(
@@ -68,9 +103,9 @@ test_that("Document section symbol works", {
     client %>% did_save(defn_file)
     result <- client %>% respond_document_symbol(defn_file)
 
-    expect_equal(
-        result %>% map_chr(~ .$name) %>% sort(),
-        c("section1", "f", "step1", "step2", "section2", "g", "p", "m") %>% sort()
+    expect_setequal(
+        result %>% map_chr(~ .$name),
+        c("section1", "f", "step1", "step2", "section2", "g", "p", "m")
     )
     expect_equivalent(
         result %>% detect(~ .$name == "section1") %>% pluck("location", "range"),
@@ -135,27 +170,25 @@ test_that("Workspace Symbol works", {
     client %>% did_save(defn_file)
     client %>% did_save(defn2_file)
 
-    expected_names <- c("f1", "f2") %>% sort()
+    expected_names <- c("f1", "f2")
     result <- client %>% respond_workspace_symbol(
         query = "f",
         retry_when = function(result) length(result) < 2
     )
 
     result_names <- result %>%
-        map_chr(~ .$name) %>%
-        sort()
-    expect_equal(result_names, expected_names)
+        map_chr(~ .$name)
+    expect_setequal(result_names, expected_names)
 
-    expected_names <- c("p1", "p2") %>% sort()
+    expected_names <- c("p1", "p2")
     result <- client %>% respond_workspace_symbol(
         query = "p",
         retry_when = function(result) length(result) < 2
     )
 
     result_names <- result %>%
-        map_chr(~ .$name) %>%
-        sort()
-    expect_equal(result_names, expected_names)
+        map_chr(~ .$name)
+    expect_setequal(result_names, expected_names)
 })
 
 test_that("Document section symbol works in Rmarkdown", {
@@ -218,13 +251,13 @@ test_that("Document section symbol works in Rmarkdown", {
     client %>% did_save(defn_file)
     result <- client %>% respond_document_symbol(defn_file)
 
-    expect_equal(
-        result %>% map_chr(~ .$name) %>% sort(),
+    expect_setequal(
+        result %>% map_chr(~ .$name),
         c("section1", "subsection1", "unnamed-chunk-1",
             "f", "section2", "unnamed-chunk-2", "g",
             "unnamed-chunk-3", "chunk1", "p", "chunk1a", "chunk2", "chunk2a",
             "chunk3", "chunk3a", "chunk4, new", "unnamed-chunk-4"
-        ) %>% sort()
+        )
     )
     expect_equivalent(
         result %>% detect(~ .$name == "section1") %>% pluck("location", "range"),
